from unittest import TestCase

from opencnam import InvalidPhoneNumberError, Phone


class PhoneTest(TestCase):

    def test_init_requires_number(self):
        self.assertRaises(TypeError, Phone)

#    def test_init_attempts_to_lookup_cnam(self):
#       TODO: Mock and make sure that Phone.get_cnam() was called one time.

    def test_init_sets_number_after_validation(self):
        phone = Phone('8182179229')
        self.assertEqual(phone.number, '8182179229')

    def test_clean_removes_non_ints(self):
        phone = Phone('8182179229a')
        self.assertEqual(phone.number, '8182179229')

        phone = Phone('a8182179229')
        self.assertEqual(phone.number, '8182179229')

        phone = Phone('   818   217   -9229')
        self.assertEqual(phone.number, '8182179229')

    def test_clean_raises_error_on_short_number(self):
        self.assertRaises(InvalidPhoneNumberError, Phone, '818217922')

    def test_clean_only_last_10_digits_of_number(self):
        phone = Phone('11111111111111118182179229')
        self.assertEqual(phone.number, '8182179229')

    def test_clean_returns_valid_number_untouched(self):
        phone = Phone('8182179229')
        self.assertEqual(phone.number, '8182179229')

    def test_clean_catches_invalid_numbers(self):
        self.assertRaises(InvalidPhoneNumberError, Phone, '1001000000')
        self.assertRaises(InvalidPhoneNumberError, Phone, '2001999999')

#    def test_get_cnam_doesnt_lookup_cnam_if_cnam_is_already_set(self):
#       TODO: Mock and make sure that once Phone.cnam is set, calls to
#       Phone.get_cnam() don't make any API calls and use the cached cnam
#       instead.

#   def test_get_cnam_sets_cnam(self):
#       TODO: Mock and make sure that once Phone.get_cnam() is called, it
#       actually updates Phone.cnam with the value.

#   def test_get_cnam_doesnt_do_anything_on_error(self):
#       TODO: Mock and make sure that if the API call fails to opencnam, that
#       Phone.get_cnam() doesn't do anything and leaves Phone.cnam alone.



"""A simple python library for getting caller ID name information."""


from re import sub

from requests import get
from requests.exceptions import ConnectionError

from .errors import InvalidPhoneNumberError


class Phone(object):
    """An object that holds a single phone's information.

    :attr str OPENCNAM_API_URL: Current OpenCNAM API endpoint.
    :attr str number: The validated 10-digit US phone number.
    :attr str cnam: The caller ID name for this phone.
    :attr str api_user: Your API username.
    :attr str api_key: Your API key.
    """
    OPENCNAM_API_URL = 'https://api.opencnam.com/v1/phone/%s'

    def __init__(self, number, cnam='', api_user=None, api_key=None):
        """Create a new Phone object, and attempt to lookup the caller ID name
        information using opencnam's public API.

        :param unicode number: The phone number to query in any format.
        :param unicode cnam: If you'd like to manually set the caller ID name
            for this phone number, you can do so here.
        :param str api_user: Your API username.
        :param str api_key: Your API key.

        Usage::

            from opencnam import Phone

            phone = Phone('+18182179229')
            ...

        ..note::
            When a new Phone object is created, we'll attempt to clean up the
            phone number input to convert it into a usable format for use with
            opencnam's API.
        """
        self.cnam = unicode(cnam)
        self.number = unicode(number)
        self.api_user = api_user
        self.api_key = api_key

        # Clean up ``number``, and try to build a valid phone number that
        # opencnam can use.
        self.clean()

        # Attempt to grab the caller ID name information from opencnam. If we
        # can't get a caller ID response back, we won't retry (developers can
        # manually retry to grab the caller ID at any time using ``get_cnam``).
        self.get_cnam()

    def clean(self):
        """Clean up ``number``, trying to make it a valid phone number that
        opencnam can use. opencnam only supports 10-digit US phone numbers with
        no symbols.

        :raises: InvalidPhoneNumberError if ``number`` cannot be made into a
            valid US phone number.

        Usage::

            from opencnam import Phone

            phone = Phone('+1-818-217-9229')
            assert phone.number == '8182179229'
        """
        self.number = sub(r'\D', '', self.number)
        self.number = self.number[-10:]

        if len(self.number) < 10 or not (2002000000 <= long(self.number) <= 9999999999):
            raise InvalidPhoneNumberError

    def get_cnam(self):
        """Query the OpenCNAM API and retreive the caller ID name string
        associated with this phone.

        Once we've got a valid caller ID name for this phone number, we'll
        cache that name for future reference.
        """
        if not self.cnam:
            params = {'format': 'text'}

            # If the user supplied API creds, use them.
            if self.api_user and self.api_key:
                params['username'] = self.api_user
                params['api_key'] = self.api_key

            try:
                response = get(self.OPENCNAM_API_URL % self.number, params=params)
                if response.status_code == 200:
                    self.cnam = response.text
            except ConnectionError:
                pass



"""A simple python library for getting caller ID name information."""


from .errors import *
from .opencnam import *


__author__ = 'Randall Degges'
__copyright__ = 'Telephony Research Services, LLC'
__credits__ = ('Randall Degges', )

__license__ = 'UNLICENSE'
__version = '0.2'
__maintainer__ = 'Randall Degges'
__email__ = 'rdegges@gmail.com'
__status__ = 'Development'



"""OpenCNAM exceptions."""


class OpenCNAMError(Exception):
    """Generic OpenCNAM error."""
    pass


class InvalidPhoneNumberError(OpenCNAMError):
    """Phone number is invalid. Phone numbers must be at least 10-digits in
    length.
    """
    pass



from setuptools import find_packages, setup


setup(

    # Basic package information.
    name = 'opencnam',
    version = '0.3',
    packages = find_packages(),

    # Packaging options.
    zip_safe = False,
    include_package_data = True,

    # Package dependencies.
    install_requires = ['requests>=0.13.2'],

    # Metadata for PyPI.
    author = 'Randall Degges',
    author_email = 'rdegges@gmail.com',
    license = 'UNLICENSE',
    url = 'http://www.opencnam.com',
    keywords = 'voip http api rest caller id name cid cnam telephony ' \
            'telephone python library',
    description = 'A simple python library for getting caller ID name ' \
            'information using the opencnam API.',
    long_description = open('README.md').read(),
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Telecommunications Industry',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Communications',
        'Topic :: Communications :: Internet Phone',
        'Topic :: Communications :: Telephony',
        'Topic :: Internet',
        'Topic :: Utilities',
    ],

)
